#!/bin/bash
#
# name           : wrapper_murx
#
# syntax         : wrapper_murx <accounts>
#
# description    : This program can be used for automatic poll of your
#                  e-mail accounts (e.g. as a cronjob).
#
#                  It runs murx with your rcfile but also overwrites
#                  some parameters. If there are any remaining mails it
#                  runs fetchmail.
#
#                  Any activity is logged and if desired a spam report
#                  is sent to a given email address (probably simply
#                  your loginname).
#
# preparations   : - your .murxrc files must be renamed to .murxrc_<account>
#                    (I recommend only defining your account(s) first and
#                     using INCLUDEs for the filters)
#
#                  - your .fetchmailrc must be renamed to 
#                    .fetchmailrc_<account>
#
#                    -> where <account> can be any string
#                       (e.g. name of provider)
#
#                  - create a .murx_addresses file containing mail addresses
#                    from domains which could but should not be deleted and
#                    you wish to get informed if this happened accidentaly.
#                    extended regex are allowed. :-)
#
#                    This feature is important for me for two reasons:
#
#                    1) I don't use ALLOW-filters
#
#                    2) I get a lot of spam mail so the report takes too
#                       long to check manually
#
#                  ---------------------------------------------
#                  - edit the parameters of this script below! -
#                  ---------------------------------------------
#
# author         : Kai Hildebrandt <kai.hildebrandt@web.de>
#
# date           : 2006-12-28
#
# version        : 0.6
#
# license        : Licensed under the same terms as MuRX, GPL v2 or later
# -----------------------------------------------------------------------------
# BUGS/Limitations
# -----------------------------------------------------------------------------
# Please send bugs and suggestions in this script to the author.
# -----------------------------------------------------------------------------
# ChangeLog
# -----------------------------------------------------------------------------
# 26.12.2006  KH   added mail address list file for checking false positives
# 07.04.2006  KH   updated comments
# 13.03.2006  KH   check for mutt to send report as attachment
# 15.01.2006  KH   creation date
# -----------------------------------------------------------------------------
# -                  SET VARIABLES BELOW TO FIT YOUR NEEDS                    -
# -----------------------------------------------------------------------------

# use which or set path to binaries statically (e.g. /usr/bin/fetchmail)
FETCHMAIL=$(which fetchmail)
MURX=$(which murx)
# mutt is used for sending reports hence you cannot send attachments
# with the mail command. An attachment is more comfortable to handle.
# Uncomment the following line if you don't have mutt installed or you
# don't want to receive your spam report as attachment.
MUTT=$(which mutt)

# store logfiles relative to this directory
LOGDIR=${HOME}/logs

# Set the appropriate loglevel:
#
# Set the murx loglevel to 0-5 and uncomment the following lines for
# special bit behaviours
#
# murx loglevel
LOGLEVEL=5
# show headers
LOGLEVEL=$(( ${LOGLEVEL} + 8 ))
# show matches
LOGLEVEL=$(( ${LOGLEVEL} + 16 ))
# show all matches
#LOGLEVEL=$(( ${LOGLEVEL} + 32 ))
# show header line with match
LOGLEVEL=$(( ${LOGLEVEL} + 64 ))

# If you want this script to overwrite the loglevel defined in your
# rcfile with the variable LOGLEVEL defined above, uncomment the
# following line.
USE_VERBOSITY="yes"

# These are the full-path prefixes for the fetchmailrc its log.
#
# For each account the string _<account> is added, e.g. if an
# account name is example_com then the script looks for the rcfile
# at .fetchmailrc_example_com in the users home directory.
#
FETCHMAILRC_BASE=${HOME}/.fetchmailrc
FETCHMAIL_LOGFILE_BASE=${LOGDIR}/fetchmail

# Contains a space separated list with account names for
# which fetchmail should not be called after filtering.
# This is intended for use with IMAP accounts.
# Use "*" for all accounts.
#NO_FETCH_ACCOUNTS="*"

# These are the full-path prefixes for the rc and logfiles.
#
# For each account the string _<account> is added, e.g. if an
# account name is example_com then the script looks for the rcfile
# at .murxrc_example_com in the users home directory.
#
MURXRC_BASE="${HOME}/.murxrc"
MURX_LOGFILE_BASE="${LOGDIR}/murx"
MURX_HEADERFILE_BASE="${LOGDIR}/headers"

# If you want this script to define its own logfile and headerfile
# names via the command line options from murx, uncomment the following
# line.
USE_LOGFILE_BASE="yes"

# A list of mail addresses to check for in From-line.
# This is used to check your filtered messages for false positives and should contain
# all e-mail addresses from your friends. The file can also contain extended regular
# expressions.
MAIL_ADDRESSES="${HOME}/.murx_addresses"

# If not empty a report will be sent to this address
# in case of errors, warnings and deleted mails.
# If this script is called with the option -v (verbose)
# a report is sent in any case.
#
# Use ${LOGNAME} if you have a running mail server on the machine.
MAILTO=${LOGNAME}

# -----------------------------------------------------------------------------
# -                DO NOT CHANGE ANYTHING BELOW THIS LINE                     -
# -----------------------------------------------------------------------------

# do some checks before start
if [ -z "${FETCHMAIL}" ]; then
    echo "${ME}: binary for fetchmail not found. Please set the variable FETCHMAIL manually."
    exit 2
fi

if [ -z "${MURX}" ]; then
    echo "${ME}: binary for murx not found. Please set the variable MURX manually."
    exit 2
fi


# that's my name
ME=$(basename $0)

# make sure those parameters were set
LOGLEVEL=${LOGLEVEL:=5}

# report files (temporary)
REPORT1=/var/tmp/report_wrapper_murx_$$.1.tmp
REPORT2=/var/tmp/report_wrapper_murx_$$.2.tmp
# for false positive checking
REPORT3=/var/tmp/report_wrapper_murx_$$.3.tmp

if [ -e "${REPORT1}" ]; then
    echo "${ME}: file ${REPORT1} exists. Abort."
    exit 2
elif [ -e "${REPORT2}" ]; then
    echo "${ME}: file ${REPORT2} exists. Abort."
    exit 2
fi

# egrep patterns
pat_error="^murx: error:"
pat_warning=".{16}warning:"
pat_deleted="(not )?deleted"

# init values
args=""
murx_args=""
fetchmail_args="-t 30"
do_fetchmail="yes"
testing="no"
verbose="no"



#############
# functions #
#############

check_false_positives()
{
    if [ x"${MAIL_ADDRESSES}" = x ]; then
	return
    fi

    echo "*** found probable false positives:" > "${REPORT3}"
    if [ -f "${MAIL_ADDRESSES}" ]; then
	while read address; do
	    #echo "DEBUG: address=\"${address}\""
	    cut -b17- "${REPORT1}" | egrep -e "^${pat_deleted}" | egrep -i -e "${address}" >> "${REPORT3}"
	done < "${MAIL_ADDRESSES}"
	if [ $( cat "${REPORT3}" | wc -l ) -gt 1 ]; then
	    echo -e "*** false positive check ends here\n" >> "${REPORT3}"
	else
	    rm -f "${REPORT3}"
	fi
    else
	echo "${ME}: file ${MAIL_ADDRESSES} not found or unreadable."
    fi
}

clean_exit()
{
    if [ -f "${REPORT1}" ]; then
	rm -f "${REPORT1}"
    fi

    if [ -f "${REPORT2}" ]; then
	rm -f "${REPORT2}"
    fi

    if [ -f "${REPORT3}" ]; then
	rm -f "${REPORT3}"
    fi

    # restore LANG
    LANG=${OLDLANG}
}


filter_mail()
{
    local MURXRC="${MURXRC_BASE}_$1"
    local LOGFILE="${MURX_LOGFILE_BASE}_${1}.log"
    local HEADERFILE="${MURX_HEADERFILE_BASE}/headers-${1}-$(date +%Y-%m)"

    # set verbosity
    if [ "${USE_VERBOSITY}" = "yes" ]; then
	murx_args="${murx_args} --verbose=${LOGLEVEL}"
    fi

    if [ "${USE_LOGFILE_BASE}" = "yes" ]; then
	murx_args="${murx_args} --logfile=${LOGFILE} --headerfile=${HEADERFILE}"
    fi

    if [ -f "${MURXRC}" ]; then
	${MURX} ${murx_args} -M "${MURXRC}" >> "${REPORT1}" 2>&1

	# get mail count
	# note: if the first message causes a parser error, the message count will be in line 3 instead of 2
	msg_count=$( head -3 "${REPORT1}" | grep "messages" | cut -b17- | sed -re "s/^querying account \".+\" \((.+) messages\)/\1/" )

	# be sure, that msg_count is set
	msg_count=${msg_count:-0}

	# set flags
	if [ $( egrep "${pat_error}" "${REPORT1}" | wc -l ) -gt 0 ]; then
	    was_error=1
	elif [ $( egrep "${pat_warning}" "${REPORT1}" | wc -l ) -gt 0 ]; then
	    was_warning=1
	else
	    msg_deleted=$( cut -b17- "${REPORT1}" | egrep "${pat_deleted}" | wc -l )

	    # be sure, that msg_deleted is set
	    msg_deleted=${msg_deleted:-0}

	    if [ ${msg_deleted} -gt 0 ]; then
		was_deleted=1
		msg_remaining=$(( ${msg_count} - ${msg_deleted} ))
	    else
		msg_remaining=$(( ${msg_count} ))
	    fi
	fi

    else
	echo "${ME}: warning: file ${MURXRC} not found. Skipping murx."
    fi
}


fetch_mail()
{
    local FETCHMAILRC="${FETCHMAILRC_BASE}_$1"
    local FETCHMAIL_LOGFILE="${FETCHMAIL_LOGFILE_BASE}_${1}.log"

    if [ "${do_fetchmail}" = "yes" ]; then
	if [ ${msg_remaining} -gt 0 -o "${testing}" = "yes" -a ${msg_count} -gt 0 ]; then
	    if [ -f "${FETCHMAILRC}" ]; then
		echo -e "\nfetchmail run by ${ME} on $(date +%d.%m.%Y) at $(date +%H:%M:%S):" >> "${FETCHMAIL_LOGFILE}"
		${FETCHMAIL} ${fetchmail_args} -f "${FETCHMAILRC}" >> "${FETCHMAIL_LOGFILE}" 2>&1
	    else
		echo "${ME}: warning: file ${FETCHMAILRC} not found. Skipping fetchmail."
	    fi
	fi
    fi
}


filter_loop()
{
    local murx_args_backup=${murx_args}

    while [ $# -gt 0 ]; do

	# init values before filtering
	murx_args=${murx_args_backup}

	was_error=0
	was_warning=0
	was_deleted=0
	msg_count=0
	msg_deleted=0
	msg_remaining=0
	
	filter_mail $1

	check_false_positives $1

	send_report $1

	if [ $( echo ${NO_FETCH_ACCOUNTS} | grep -e "$1" -e "\*" | wc -l ) -eq 0 ]; then
	    fetch_mail $1
	fi

	shift
    done
}


parse_args()
{
    while [ $# -gt 0 ]; do

	case "$1" in
	    -h|-help|--help) cat >&2 <<EOF
Usage: $ME [options] {account1} [account2] ..

General options:
  -h|--help             show this help and exit
  -n|--no-fetchmail     don't run fetchmail after filtering
  -t|--test             run murx in test mode i.e. don't delete mails
  -v|--verbose          be verbose (send full report)

EOF
		exit 0
		;;

	    -n|--no-fetchmail) do_fetchmail="no" ;;

	    -t|--test) murx_args="$murx_args -t"; testing="yes" ;;

	    -v|--verbose) fetchmail_args="$fetchmail_args -v"; verbose="yes" ;;

	    -*) echo "unknown option: $1"; usage ;;

	    *) args="$args $1" ;;
	esac
	shift
    done
}


send_report()
{
    local mail_subject="spamreport (account $1): "

    if [ ${was_error} -eq 1 ]; then
	mail_subject="${mail_subject} murx finished with errors"
    elif [ ${was_warning} -eq 1 ]; then
	mail_subject="${mail_subject} murx finished with warnings"
    elif [ ${was_deleted} -eq 1 ]; then
	mail_subject="${mail_subject} ${msg_deleted} of ${msg_count} were deleted by murx"
    else
	mail_subject="${mail_subject} filtering finished"
    fi

    # verbose mode, error or warning occured: send complete output
    if [ ${was_error} -eq 1 ]; then
	mail -e -s "${mail_subject}" ${MAILTO} < "${REPORT1}"
    elif [ ${was_warning} -eq 1 ]; then
	cut -b17- "${REPORT1}" | mail -e -s "${mail_subject}" ${MAILTO}
    elif [ "${verbose}" = "yes" ]; then
	# report false positives
	cat "${REPORT3}" > "${REPORT2}" 2>/dev/null
	# remove date
	cut -b17- "${REPORT1}" >> "${REPORT2}"
	# send report as attachment if possible (mutt)
	if [ -z "${MUTT}" ]; then
	    mail -e -s "${mail_subject}" ${MAILTO} < "${REPORT2}"
	else
	    cat <<EOF | ${MUTT} -a "${REPORT2}" -s "${mail_subject}" ${MAILTO}
Either verbose option was set or some warnings occured while filtering
your mail accounts with MuRX.

For details see attached spam report.

EOF
	fi
    else
	# only send report if there were deleted messages
	if [ ${was_deleted} -eq 1 ]; then
	    # report false positives
	    cat "${REPORT3}" > "${REPORT2}" 2>/dev/null
            # remove date, only show important lines
	    cut -b17- "${REPORT1}" \
		| egrep -e "^${pat_error}" -e "^${pat_warning}" -e "^${pat_deleted}" \
		>> "${REPORT2}"
	    # send report
	    cat "${REPORT2}" | mail -e -s "${mail_subject}" ${MAILTO}
	fi
    fi
}


usage()
{
    echo "usage: $ME [options] {account1} [account2] .."
    exit 1
}



########
# main #
########

parse_args "$@"

# save lang for restore
OLDLANG=${LANG}

trap "clean_exit" EXIT

unset LANG

if [ -z "$args" ]; then
    echo "${ME}: no accounts given."
    exit 3
fi

filter_loop $args
